//package example;

import jade.core.*;
import jade.core.behaviours.*;
import jade.lang.acl.ACLMessage;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.DFService;
import jade.domain.FIPAException;

/**
This agent implements a simple Ping Agent for the AgentCities project.
First of all the agent registers itself with the DF of the platform and 
then waits for ACLMessages.
If  a QUERY_REF message arrives that contains the string "ping" within the content 
then it replies with an INFORM message whose content will be the string "alive". 
If it receives a NOT_UNDERSTOOD message no reply is sent. 
For any other message received it replies with a NOT_UNDERSTOOD message.
The exchanged message are written in a log file whose name is the local name of the agent.

@author Tiziana Trucco - CSELT S.p.A.
@version  $Date: 2002-08-02 10:10:01 +0200 (ven, 02 ago 2002) $ $Revision: 3320 $  
*/


public class PingAgent extends Agent {

    //Logger logFile;
    
  class WaitPingAndReplyBehaviour extends SimpleBehaviour {
	
  	private boolean finished = false;
	private int i;
	
	public WaitPingAndReplyBehaviour(Agent a) {
	    super(a);
	    i = 1;
	}
	
	public void action() {
	    
		AID agentReceiver = new AID();
		//Emulator and JADE
//		agentReceiver.setName("AGENTEEMULATOR@Andromeda");
//		agentReceiver.addAddresses("http://127.0.0.1:9090/");
		
		//Phone and JADE
		agentReceiver.setName("AGENTEHTCDESIRE@Andromeda");
		agentReceiver.addAddresses("http://158.42.15.166:9090/");
		//agentReceiver.addAddresses("http://192.168.1.112:9090/");
		
		//Beagleboard and JADE
//		agentReceiver.setName("BEAGLEBOARD@Andromeda");
//		agentReceiver.addAddresses("http://158.42.186.113:9090/");
		
		//solo la primera vez 
		if (i==1){
			ACLMessage  msg = new ACLMessage(ACLMessage.INFORM);
		    msg.setContent("Start of JADE Agent");
		    msg.setSender(getAID());
		    //msg.addReplyTo(getAID());
		    msg.addReceiver(agentReceiver);
		    send(msg);
		    i++;
		    System.out.println("mensaje enviado");
		}
		
	    
	    System.out.println("esperando recibir");
	    ACLMessage  msg=null;
	   
	    do{
	      msg = blockingReceive();
	      //System.out.println("esperando");
	    }while(msg ==null);
	    	
	    if(msg != null){
		if(msg.getPerformative() == ACLMessage.NOT_UNDERSTOOD){
			//received a NOT-UNDERSTOOD message
			System.out.println("No reply message sent");
		    }
		else{
		    
		    ACLMessage reply = msg.createReply();
		    
		    //if((msg.getPerformative()== ACLMessage.QUERY_REF)||(msg.getPerformative()== ACLMessage.QUERY_IF))
		    if(msg.getPerformative()== ACLMessage.INFORM){
		
				String content = msg.getContent();
				if ((content != null) && (content.indexOf("ping") != -1)){
				    //received a QUERY_REF with correct content.
				    System.out.println("LLego el mensaje de: " + msg.getSender().getName() + "  " + msg.getContent());
				    reply.setPerformative(ACLMessage.INFORM);
				    //reply.setContent("(pong)");
				    reply.setContent("alive");
				}else if ((content != null) && (content.indexOf("pong") != -1)){
				    //received a QUERY_REF with correct content.
				    System.out.println("LLego el mensaje de: " + msg.getSender().getName() + "  " + msg.getContent());
				}
				else{
				    //received a QUERY_REF with uncorrect content.
				    System.out.println(msg.getSender().getName() + ACLMessage.getPerformative(msg.getPerformative()));
				    reply.setPerformative(ACLMessage.NOT_UNDERSTOOD);
				    reply.setContent("( UnexpectedContent (expected ping))");
				    //write the full message.
				    System.out.println(((AID)reply.getAllReceiver().next()).getName() + ACLMessage.getPerformative(reply.getPerformative()));
				}
		    }
		    else {
			//received a wrong performative.
			System.out.println(msg.getSender().getName() + ACLMessage.getPerformative(msg.getPerformative()));
			reply.setPerformative(ACLMessage.NOT_UNDERSTOOD);
			reply.setContent("( (Unexpected-act "+ACLMessage.getPerformative(msg.getPerformative())+") ( expected (query-ref :content ping)))");   
			System.out.println(((AID)(reply.getAllReceiver().next())).getName() + ACLMessage.getPerformative(reply.getPerformative()));
		    }
		    
		  
		    send(reply);
		}
	    }else{
		System.out.println("No message received");
	    }
	}
	
	public boolean done() {
		 return finished;
	}
    } //End class WaitPingAndReplyBehaviour
    
    
    protected void setup() {
  	
    this.getAID().toString();
   // this.getAID().addAddresses();
    	
	/** Registration with the DF */
	DFAgentDescription dfd = new DFAgentDescription();
	ServiceDescription sd = new ServiceDescription();   
	sd.setType("AgentcitiesPingAgent"); 
	sd.setName(getName());
	sd.setOwnership("TILAB");
	//sd.addOntologies("PingAgent");
	dfd.setName(getAID());
	dfd.addServices(sd);
	try {
	    DFService.register(this,dfd);
	} catch (FIPAException e) {
	    System.err.println(getLocalName()+" registration with DF unsucceeded. Reason: "+e.getMessage());
	    doDelete();
	}
	

	//logFile = new Logger(getLocalName());
	System.out.println("Agent: " + getName() + " born");
	WaitPingAndReplyBehaviour PingBehaviour = new  WaitPingAndReplyBehaviour(this);
	addBehaviour(PingBehaviour);
	
    }
    
   
    
}//end class PingAgent
