//Seller (Agent) that provides products and 
// expects the buyer (mobile agent) to make a counter offer that can accept or not.

import jade.core.*;
import jade.core.behaviours.*;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.DFService;
import jade.domain.FIPAException;

public class Seller extends Agent {
    
  class SellerBehaviour extends SimpleBehaviour {
	
  	private boolean finished = true;
	private int i;
	
	public SellerBehaviour(Agent a) {
	    super(a);
	    i = 1;
	}
	
	public void action() {
	    
		ACLMessage  msg=null;
        AID agentReceiver= null;
        
		System.out.println("Waiting Buyer Agent!!!");
        
		//only the first time (in cycliBehaviour)
		if (i==1){
			MessageTemplate template = MessageTemplate.MatchProtocol("START_BUY");
			do{
				msg = blockingReceive(template,5000);
				//System.out.println("Wait");
			}while(msg ==null);

			agentReceiver= msg.getSender();
			
			i++;
			System.out.println("Msg of Start receive!!!!");
			//System.out.println(msg.toString());
		    
			//agent OK, start the buy and sell
			msg = msg.createReply();
		    msg.setContent("OK");
	        msg.addReceiver(agentReceiver);
		    send(msg);
		}
		
	    //Send the items to sell
		ACLMessage  msg1 = new ACLMessage(ACLMessage.INFORM);
	    msg1.setSender(getAID());   
        msg1.setContent("[NetBook, Tablet, Game Console]:-400.00");
        msg1.setProtocol("BUY_PROTOCOL");
        msg1.addReceiver(agentReceiver);
	    send(msg1);
	    
	    System.out.println("Send the msg of items");
	   
	    //Wait the second offer of the client
	    MessageTemplate template = MessageTemplate.MatchProtocol("SECOND_ROUND_BID");
		do{
			msg = blockingReceive(template,2000);
			//System.out.println("Wait");
		}while(msg ==null);
		
		System.out.println("Receive SECOND_ROUND_BID");
		
		//The client (buyer) sent a new combined offers, the  
		//seller agent decides whether the offer is accepted or not

		//PROCESS ....
		msg1 = blockingReceive(2000); //wait 2Seg ... processing the answer (simulated)
		
		//and the agent send the answer msg
		msg= msg.createReply();
		msg.setContent("You win, your offer is accepted");
        msg.addReceiver(agentReceiver);
	    send(msg);
	    
	    System.out.println("sending the acceptance of the new offer");
        
	}
	
	public boolean done() {
		 return finished;
	}
    } //End class Behaviour
    
    
  protected void setup() {

	  this.getAID().toString();

	  /** Registration with the DF */
	  DFAgentDescription dfd = new DFAgentDescription();
	  ServiceDescription sd = new ServiceDescription();   
	  sd.setType("AgentcitiesSeller"); 
	  sd.setName(getName());
	  sd.setOwnership("DSIC");
	  dfd.setName(getAID());
	  dfd.addServices(sd);
	  try {
		  DFService.register(this,dfd);
	  } catch (FIPAException e) {
		  System.err.println(getLocalName()+" registration with DF unsucceeded. Reason: "+e.getMessage());
		  doDelete();
	  }


	  System.out.println("Agent: " + getName() + " born");
	  SellerBehaviour Seller = new  SellerBehaviour(this);
	  addBehaviour(Seller);

  }
    
   
    
}//end class Agent
